package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.uc.api.service.EmployeeService;
import com.chelvc.cloud.vehicle.api.constant.CategoryType;
import com.chelvc.cloud.vehicle.api.constant.MerchantStatus;
import com.chelvc.cloud.vehicle.api.dto.CategoryDTO;
import com.chelvc.cloud.vehicle.api.dto.GoodsDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDTO;
import com.chelvc.cloud.vehicle.api.dto.MerchantDetailDTO;
import com.chelvc.cloud.vehicle.api.param.*;
import com.chelvc.cloud.vehicle.server.copier.GoodsCopier;
import com.chelvc.cloud.vehicle.server.copier.MerchantCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantMapper;
import com.chelvc.cloud.vehicle.server.entity.Merchant;
import com.chelvc.cloud.vehicle.server.service.CategoryService;
import com.chelvc.cloud.vehicle.server.service.GoodsService;
import com.chelvc.cloud.vehicle.server.service.MerchantService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 商家业务操作实现
 *
 * @author Woody
 * @date 2023/5/2
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.MerchantService.class)
public class MerchantServiceImpl extends ServiceImpl<MerchantMapper, Merchant> implements MerchantService,
        com.chelvc.cloud.vehicle.api.service.MerchantService {
    private final GoodsService goodsService;

    private final CategoryService categoryService;

    private final EmployeeService employeeService;

    @Override
    public List<MerchantDTO> listNearbyMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listNearbyMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listRecommendMerchants(@NonNull LocationQueryParam param, int size) {
        return this.baseMapper.listRecommendMerchants(param, size);
    }

    @Override
    public List<MerchantDTO> listSimpleMerchants(@NonNull MerchantQueryParam param) {
        return this.baseMapper.listSimpleMerchants(param);
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(@NonNull Long id) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                ResourceUtils.required(this.getById(id), "商家不存在")
        );

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
//        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public MerchantDetailDTO getMerchantDetail(Long id, LocationQueryParam param) {
        // 获取商家信息
        MerchantDTO merchant = MerchantCopier.INSTANCE.copying(
                ResourceUtils.required(this.baseMapper.getMerchantById(id, param), "商家不存在")
        );
        Long userId = SessionContextHolder.getSession().getId();
        //查询商家是否被收藏
        Integer count = this.baseMapper.merchantFavorite(id,userId);
        if (count > 0){
            merchant.setFavorites(true);
        }

        // 获取商家全部商品列表
        List<GoodsDTO> goods = this.goodsService.listMerchantSimpleGoods(
                id, GoodsQueryParam.builder().size(10).build()
        );

        // 获取商家商品分类类型列表
//        List<CategoryType> types = this.goodsService.listMerchantGoodsCategoryTypes(id);
        List<Long> idSet = goods.stream()
                .map(GoodsDTO::getCategoryId) // 将每个Person对象映射到其id
                .collect(Collectors.toList());
        List<CategoryDTO> categories = this.categoryService.listCategories(idSet);
        return MerchantDetailDTO.builder().merchant(merchant).goods(goods).categories(categories).build();
    }

    @Override
    public Long addMerchant(@NonNull MerchantModifyParam param) {
        // 判断商家名称是否存在
        Integer count = this.lambdaQuery()
                .eq(Merchant::getName, param.getName())
                .eq(Merchant::getStatus, MerchantStatus.ONLINE).count();
        ResourceUtils.available(count < 1 , "商家名称已存在");
        Merchant merchant = MerchantCopier.INSTANCE.copying(param);
        merchant.setUserId(0L);
        merchant.setScore(0D);
        merchant.setSale(0);
        merchant.setStatus(MerchantStatus.ONLINE);
        this.save(merchant);
        return merchant.getId();
    }

    @Override
    public void updateMerchant(@NonNull Long id, @NonNull MerchantModifyParam param) {
        Merchant merchant = ResourceUtils.required(this.getById(id), "商家不存在");
        MerchantCopier.INSTANCE.copying(param, merchant);
        this.updateById(merchant);
    }

    @Override
    public void deleteMerchant(@NonNull Long id) {
         ResourceUtils.required(this.getById(id), "商家不存在");
        this.baseMapper.deleteById(id);
    }

    @Override
    public Pagination<MerchantDTO> getMerchantPaging(@NonNull MerchantPagingParam param) {
        Long userId = SessionContextHolder.getSession().getId();
        if (employeeService.isAdmin(userId)){
            Page<Merchant> page = this.lambdaQuery()
                    .like(StringUtils.nonEmpty(param.getName()), Merchant::getName, param.getName())
                    .eq(Objects.nonNull(param.getRecommend()), Merchant::getRecommend, param.getRecommend())
                    .eq(Objects.nonNull(param.getStatus()), Merchant::getStatus, param.getStatus())
                    .orderByDesc(Merchant::getCreateTime).page(PagingUtils.convert(param.getPaging()));
            return PagingUtils.convert(page, MerchantCopier.INSTANCE::copying);
        }else{
            Page<Merchant> page = this.lambdaQuery()
                    .like(StringUtils.nonEmpty(param.getName()), Merchant::getName, param.getName())
                    .eq(Objects.nonNull(param.getRecommend()), Merchant::getRecommend, param.getRecommend())
                    .eq(Objects.nonNull(param.getStatus()), Merchant::getStatus, param.getStatus())
                    .eq(Merchant::getUserId,userId)
                    .orderByDesc(Merchant::getCreateTime).page(PagingUtils.convert(param.getPaging()));
            return PagingUtils.convert(page, MerchantCopier.INSTANCE::copying);
        }

    }

    @Override
    public MerchantDTO getMerchant(@NonNull Long id) {
        Merchant merchant = ResourceUtils.required(this.getById(id), "商家不存在");
        return this.convert(merchant);
    }

    /**
     * 转换商家信息
     *
     * @param merchant 商家实例
     * @return 商家信息
     */
    private MerchantDTO convert(Merchant merchant) {
        return MerchantCopier.INSTANCE.copying(merchant);
    }


    @Override
    public List<Long> getMerchantIdsByUserId(Long userId){
       List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
               .list();
       return merchants.stream().map(Merchant::getId).collect(Collectors.toList());
    }
    @Override
    public List<MerchantDTO> getMerchantIdsByUserId(){
        Long userId = SessionContextHolder.getSession().getId();
        List<Merchant> merchants = this.lambdaQuery()
                .eq(Merchant::getUserId, userId)
                .eq(Merchant::getStatus, MerchantStatus.ONLINE)
                .list();
        return MerchantCopier.INSTANCE.copying(merchants);
    }
}
